from typing import Any

from graphql import GraphQLResolveInfo

from .settings import api_settings
from .utils import convert_kwargs_to_snake_case


class LoginRequiredMixin:
    """Verify that the current user is authenticated."""

    def __call__(self, parent: Any, info: GraphQLResolveInfo, **kwargs):
        user = info.context["request"].user

        if user and user.is_authenticated:
            # noinspection PyUnresolvedReferences
            return super().__call__(parent, info, **kwargs)
        else:
            raise api_settings.LOGIN_REQUIRED_ERROR


class ConvertKwargsToSnakeCaseMixin:
    """Convert the kwargs to snake_case."""

    @convert_kwargs_to_snake_case
    def __call__(self, *args, **kwargs):  # pragma: no cover
        return super().__call__(*args, **kwargs)
