from typing import List, Optional

from django.db.models import QuerySet

from .edges import Edge
from ariadne_django_resolvers.utils import apply_cursor_to_queryset


class PageInfo:
    def __init__(
        self,
        qs: QuerySet,
        edges: List[Edge],
        before: Optional[int] = None,
        after: Optional[int] = None,
    ):
        self._qs = qs
        self._edges = edges
        self._after = after
        self._before = before

    @property
    def has_previous_page(self):
        if self._edges:
            return apply_cursor_to_queryset(
                qs=self._qs, before=self._edges[0].node.pk
            ).exists()
        else:
            if self._after:
                return apply_cursor_to_queryset(
                    qs=self._qs, before=self._after,
                ).exists()
            else:
                return False

    @property
    def has_next_page(self):
        if self._edges:
            return apply_cursor_to_queryset(
                qs=self._qs, after=self._edges[-1].node.pk
            ).exists()
        else:
            if self._before:
                return apply_cursor_to_queryset(
                    qs=self._qs, after=self._before,
                ).exists()
            else:
                return False

    @property
    def start_cursor(self):
        if self._edges:
            return self._edges[0].node.pk
        else:
            return None

    @property
    def end_cursor(self):
        if self._edges:
            return self._edges[-1].node.pk
        else:
            return None
