from typing import List

from django.core.exceptions import ValidationError, NON_FIELD_ERRORS

from ..base import Resolver
from ...responses import MutationResponse


class MutationResolver(Resolver):
    errors: List[dict]
    data: dict

    def __call__(self, *args, **kwargs):
        self.errors = []
        return super().__call__(*args, **kwargs)

    def resolve(self) -> MutationResponse:
        return MutationResponse(errors=self.errors, **(self.mutate() or {}))

    def mutate(self):
        raise NotImplementedError()

    def add_error(self, msg: str, field: str = None):
        self.errors.append({"msg": msg, "field": field})

    def add_validation_error(self, validation_error: ValidationError):
        if hasattr(validation_error, "error_dict"):
            for field, error_list in validation_error.error_dict.items():
                for error in error_list:
                    self.add_error(**self._format_validation_error(error, field=field))
        else:
            for error in validation_error.error_list:
                self.add_error(**self._format_validation_error(error))

    @staticmethod
    def _format_validation_error(
        validation_error: ValidationError, field: str = None
    ) -> dict:
        field = validation_error.code or field
        msg = validation_error.message

        if validation_error.params:
            msg %= validation_error.params

        return {"msg": msg, "field": field if field != NON_FIELD_ERRORS else None}
