from django.core.exceptions import ValidationError
from django.db.models import Model


# noinspection PyUnresolvedReferences
class UpdateOrCreateMixin:
    object: Model
    object_field_name: str
    payload_variable_name = "payload"

    def mutate(self):
        self.object = self.get_object()
        self.set_attributes()
        self.clean_and_save_object()
        return {self.object_field_name: self.object}

    def set_attributes(self, payload: dict = None):
        payload = payload or self.variables[self.payload_variable_name]

        for key, value in payload.items():
            setattr(self.object, key, value)

    def clean_and_save_object(self):
        """
        Full clean the object, if ValidationErrors get raised add them to the resolers
        error list, else save the object.
        """

        if not self.errors:
            try:
                self.object.full_clean()
            except ValidationError as e:
                self.add_validation_error(e)
            else:
                self.object.save()
