from base64 import b64encode, b64decode

from ariadne import ScalarType
from graphql.pyutils import inspect

from .settings import api_settings

cursor_scalar = ScalarType("Cursor")


@cursor_scalar.serializer
def serialize_cursor(value: int) -> str:
    value = api_settings.SCHEMA.get_type("ID").serialize(value)
    return b64encode(f"arrayconnection:{value}".encode()).decode()


@cursor_scalar.value_parser
def parse_cursor_value(value: str) -> int:
    if not isinstance(value, str):
        raise TypeError(f"Cursor cannot represent non string value: {inspect(value)}")
    value = b64decode(value).decode().split(":")[1]
    value = api_settings.SCHEMA.get_type("ID").parse_value(value)
    return value
