from math import inf

from django.core.exceptions import PermissionDenied
from django.utils.module_loading import import_string

from django.conf import settings

DEFAULTS = {
    "MAX_PAGE_SIZE": inf if settings.DEBUG else None,
    "SCHEMA": None,
    "LOGIN_REQUIRED_ERROR": PermissionDenied("Login required."),
}

IMPORT_STRINGS = ["SCHEMA"]


class ApiSettings:
    namespace = "ARIADNE_DJANGO_RESOLVERS"

    def __getattr__(self, attr):
        if attr not in DEFAULTS:
            raise AttributeError(f"Invalid API setting: '{attr}'")

        try:
            val = getattr(settings, self.namespace, {})[attr]
        except KeyError:
            val = DEFAULTS[attr]

        if attr in IMPORT_STRINGS:
            val = import_string(val)

        return val


api_settings = ApiSettings()
